{ Copyright (C) 2012 H Page-Clark

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA. }

unit dualdigitsset;

{$mode objfpc}{$H+}

interface

uses
  SysUtils;

type
  TDigits = 0..9;
  TDigitsSet = set of TDigits;

  { TDualAZSet }

  TDualAZSet = class
    private
      FsetA, FsetZ: TDigitsSet;
      function GetDiffAZAsText: string;
      function GetDiffZAAsText: string;
      function GetIntersectionAsText: string;
      function GetSetAasText: string;
      function GetSetZasText: string;
      function GetSymDiffAsText: string;
      function GetUnionAsText: string;
      function SetAsString(s: TDigitsSet): string;
    public
      property DiffAZAsText: string read GetDiffAZAsText;
      property DiffZAAsText: string read GetDiffZAAsText;
      property IntersectionAsText: string read GetIntersectionAsText;
      property SetA: TDigitsSet read FsetA write FsetA;
      property SetAasText: string read GetSetAasText;
      property SetZ: TDigitsSet read FsetZ write FsetZ;
      property SetZasText: string read GetSetZasText;
      property SymDiffAsText: string read GetSymDiffAsText;
      property UnionAsText: string read GetUnionAsText;
  end;

implementation

{ TDualAZSet}

function TDualAZSet.GetDiffAZAsText: string;
begin
 Result:= SetAsString(FsetA - FsetZ);
end;

function TDualAZSet.GetDiffZAAsText: string;
begin
 Result:= SetAsString(FsetZ - FsetA);
end;

function TDualAZSet.GetIntersectionAsText: string;
begin
 Result:= SetAsString(FsetA*FsetZ);
end;

function TDualAZSet.GetSetAasText: string;
begin
 Result:= SetAsString(FsetA);
end;

function TDualAZSet.GetSetZasText: string;
begin
 Result:= SetAsString(FsetZ);
end;

function TDualAZSet.GetSymDiffAsText: string;
begin
 Result:= SetAsString(FsetA+FsetZ - FsetA*FsetZ);
end;

function TDualAZSet.GetUnionAsText: string;
begin
  Result:= SetAsString(FsetA + FsetZ);
end;

function TDualAZSet.SetAsString(s: TDigitsSet): string;
var d: TDigits;
begin
 Result:= EmptyStr;
 for d in TDigitsSet do
  if (d in s) then
   begin
    if Length(Result) > 0 then AppendStr(Result, ',');
    AppendStr(Result, IntToStr(d));
   end;
 Result:= Format('[%s]',[Result]);
end;

end.

